@@ -18,18 +18,13 @@ except ImportError:
 
 
 try:
-    from importlib.util import (
-        cache_from_source as get_pyc_path,
-        source_from_cache as get_py_path,
-    )
+    from importlib.util import source_from_cache as get_py_path
 except ImportError:
     if PY2:
-        get_pyc_path = lambda path: path + 'c'
         get_py_path = lambda path: path[:-1]
 
     # fallback on python < 3.5
     else:
-        get_pyc_path = imp.cache_from_source
         get_py_path = imp.source_from_cache
 
 
@@ -39,15 +34,6 @@ except ImportError:
     import pickle
 
 
-def is_watchdog_supported():
-    """ Return ``True`` if watchdog is available."""
-    try:
-        import watchdog
-    except ImportError:
-        return False
-    return True
-
-
 ################################################
 # cross-compatible metaclass implementation
 # Copyright (c) 2010-2012 Benjamin Peterson
