@@ -1,14 +1,20 @@
 from __future__ import print_function
 
 from glob import glob
+import os
 import signal
 import sys
 import threading
 import time
 
-from .compat import is_watchdog_supported
 from .compat import queue
 from .ipc import ProcessGroup
+from .logger import DefaultLogger
+from .utils import (
+    resolve_spec,
+    is_watchdog_supported,
+    is_watchman_supported,
+)
 from .worker import (
     Worker,
     is_active,
@@ -23,16 +29,13 @@ class FileMonitorProxy(object):
     when it should reload.
 
     """
-    def __init__(self, monitor_factory, verbose=1):
-        self.monitor = monitor_factory(self.file_changed)
-        self.verbose = verbose
+    monitor = None
+
+    def __init__(self, logger):
+        self.logger = logger
         self.change_event = threading.Event()
         self.changed_paths = set()
 
-    def out(self, msg):
-        if self.verbose > 0:
-            print(msg)
-
     def add_path(self, path):
         # if the glob does not match any files then go ahead and pass
         # the pattern to the monitor anyway incase it is just a file that
@@ -50,7 +53,7 @@ class FileMonitorProxy(object):
     def file_changed(self, path):
         if path not in self.changed_paths:
             self.changed_paths.add(path)
-            self.out('%s changed; reloading ...' % (path,))
+            self.logger.info('%s changed; reloading ...' % (path,))
         self.set_changed()
 
     def is_changed(self):
@@ -76,8 +79,8 @@ class Reloader(object):
     def __init__(self,
                  worker_path,
                  monitor_factory,
+                 logger,
                  reload_interval=1,
-                 verbose=1,
                  worker_args=None,
                  worker_kwargs=None,
                  ):
@@ -86,15 +89,11 @@ class Reloader(object):
         self.worker_kwargs = worker_kwargs
         self.monitor_factory = monitor_factory
         self.reload_interval = reload_interval
-        self.verbose = verbose
+        self.logger = logger
         self.monitor = None
         self.worker = None
         self.group = ProcessGroup()
 
-    def out(self, msg):
-        if self.verbose > 0:
-            print(msg)
-
     def run(self):
         """
         Execute the reloader forever, blocking the current thread.
@@ -145,7 +144,7 @@ class Reloader(object):
             # register the worker with the process group
             self.group.add_child(self.worker.pid)
 
-            self.out("Starting monitor for PID %s." % self.worker.pid)
+            self.logger.info("Starting monitor for PID %s." % self.worker.pid)
             self.monitor.clear_changes()
 
             while not self.monitor.is_changed() and self.worker.is_alive():
@@ -161,25 +160,27 @@ class Reloader(object):
                     for path in cmd[1]:
                         self.monitor.add_path(path)
 
-                else:
+                else:  # pragma: no cover
                     raise RuntimeError('received unknown command')
 
         except KeyboardInterrupt:
             if self.worker.is_alive():
-                self.out('Waiting for server to exit ...')
+                self.logger.info('Waiting for server to exit ...')
                 time.sleep(self.reload_interval)
             raise
 
         finally:
             if self.worker.is_alive():
-                self.out('Killing server with PID %s.' % self.worker.pid)
+                self.logger.info(
+                    'Killing server with PID %s.' % self.worker.pid,
+                )
                 self.worker.terminate()
                 self.worker.join()
 
             else:
                 self.worker.join()
-                self.out('Server with PID %s exited with code %d.' %
-                         (self.worker.pid, self.worker.exitcode))
+                self.logger.info('Server with PID %s exited with code %d.' %
+                                 (self.worker.pid, self.worker.exitcode))
 
         self.monitor.clear_changes()
 
@@ -188,7 +189,7 @@ class Reloader(object):
         return force_restart
 
     def _wait_for_changes(self):
-        self.out('Waiting for changes before reloading.')
+        self.logger.info('Waiting for changes before reloading.')
         while (
             not self.monitor.wait_for_change(self.reload_interval)
         ):  # pragma: nocover
@@ -197,7 +198,13 @@ class Reloader(object):
         self.monitor.clear_changes()
 
     def _start_monitor(self):
-        self.monitor = FileMonitorProxy(self.monitor_factory, self.verbose)
+        proxy = FileMonitorProxy(self.logger)
+        proxy.monitor = self.monitor_factory(
+            proxy.file_changed,
+            interval=self.reload_interval,
+            logger=self.logger,
+        )
+        self.monitor = proxy
         self.monitor.start()
 
     def _stop_monitor(self):
@@ -211,7 +218,7 @@ class Reloader(object):
             signal.signal(signal.SIGHUP, self._signal_sighup)
 
     def _signal_sighup(self, signum, frame):
-        self.out('Received SIGHUP, triggering a reload.')
+        self.logger.info('Received SIGHUP, triggering a reload.')
         self.monitor.set_changed()
 
     def _restore_signals(self):
@@ -219,6 +226,31 @@ class Reloader(object):
             signal.signal(signal.SIGHUP, signal.SIG_DFL)
 
 
+def find_default_monitor_factory(logger):
+    spec = os.getenv('HUPPER_DEFAULT_MONITOR')
+    if spec:
+        monitor_factory = resolve_spec(spec)
+
+        logger.debug('File monitor backend: ' + spec)
+
+    elif is_watchman_supported():
+        from .watchman import WatchmanFileMonitor as monitor_factory
+
+        logger.debug('File monitor backend: watchman')
+
+    elif is_watchdog_supported():
+        from .watchdog import WatchdogFileMonitor as monitor_factory
+
+        logger.debug('File monitor backend: watchdog')
+
+    else:
+        from .polling import PollingFileMonitor as monitor_factory
+
+        logger.debug('File monitor backend: polling')
+
+    return monitor_factory
+
+
 def start_reloader(
     worker_path,
     reload_interval=1,
@@ -245,7 +277,7 @@ def start_reloader(
     restarts.
 
     ``verbose`` controls the output. Set to ``0`` to turn off any logging
-    of activity and turn up to ``2`` for extra output.
+    of activity and turn up to ``2`` for extra output. Default is ``1``.
 
     ``monitor_factory`` is an instance of
     :class:`hupper.interfaces.IFileMonitorFactory`. If left unspecified, this
@@ -254,35 +286,26 @@ def start_reloader(
     fallback to the less efficient
     :class:`hupper.polling.PollingFileMonitor` otherwise.
 
+    If ``monitor_factory`` is ``None`` it can be overridden by the
+    ``HUPPER_DEFAULT_MONITOR`` environment variable. It should be a dotted
+    python path pointing at an object implementing
+    :class:`hupper.interfaces.IFileMonitorFactory`.
+
     """
     if is_active():
         return get_reloader()
 
+    logger = DefaultLogger(verbose)
+
     if monitor_factory is None:
-        if is_watchdog_supported():
-            from .watchdog import WatchdogFileMonitor
-
-            def monitor_factory(callback):
-                return WatchdogFileMonitor(callback)
-
-            if verbose > 1:
-                print('File monitor backend: watchdog')
-
-        else:
-            from .polling import PollingFileMonitor
-
-            def monitor_factory(callback):
-                return PollingFileMonitor(callback, reload_interval)
-
-            if verbose > 1:
-                print('File monitor backend: polling')
+        monitor_factory = find_default_monitor_factory(logger)
 
     reloader = Reloader(
         worker_path=worker_path,
         worker_args=worker_args,
         worker_kwargs=worker_kwargs,
         reload_interval=reload_interval,
-        verbose=verbose,
         monitor_factory=monitor_factory,
+        logger=logger,
     )
     return reloader.run()
